/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 22/07/22.
//

#ifndef FGGL_ECS_COMPONENT_FWD_HPP
#define FGGL_ECS_COMPONENT_FWD_HPP

#include "fggl/ecs/component.hpp"

#include "fggl/math/types.hpp"
#include "fggl/gfx/phong.hpp"
#include "fggl/data/model.hpp"
#include "fggl/phys/types.hpp"

#ifdef __GNUC__
	#include <cxxabi.h>
#endif

namespace fggl::ecs {

	template<typename T>
	bool restore_config(T* comp, const YAML::Node& node) {
		char* realName;

		#ifdef __GNUC__
		int status;
		realName = abi::__cxa_demangle(typeid(T).name(), 0, 0, &status);
		#endif

		debug::log(debug::Level::warning, "restore_config is not implemented for {}", realName);
		return false;
	}

	template<>
	bool restore_config(math::Transform* comp, const YAML::Node& node);

	template<>
	bool restore_config(gfx::PhongMaterial* comp, const YAML::Node& node);

	template<>
	bool restore_config(data::StaticMesh* meshComp, const YAML::Node& node);

	template<>
	bool restore_config(phys::RigidBody* body, const YAML::Node& node);

	template<>
	bool restore_config(phys::CollisionCallbacks* callbacks, const YAML::Node& node);

	template<>
	bool restore_config(phys::CollisionCache* cache, const YAML::Node& node);

} // namespace fggl::ecs

#endif //FGGL_ECS_COMPONENT_FWD_HPP
